import 'dart:async';

import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_repository/firebase_repository.dart';
import 'package:firebase_repository/src/models/descarga.dart';
import 'package:meta/meta.dart';
import 'descargas_repository.dart';

class FirebaseDescargasRepository implements DescargaRepository{
  CollectionReference descargasCollection;
  String usuarioId;
  String cargaId = '';
  // ignore: close_sinks
  StreamController<List<Descarga>> streamDescargas = StreamController<List<Descarga>>();

  FirebaseDescargasRepository({@required String empresaId, @required this.usuarioId}) {
    this.descargasCollection = Firestore.instance
        .collection('empresas')
        .document(empresaId)
        .collection("descargas");

    Firestore.instance
        .collection('empresas')
        .document(empresaId)
        .collection("cargas").where("usuario", isEqualTo: this.usuarioId)
        .where("activo", isEqualTo: true).snapshots().map((snapshot) {
      return snapshot.documents
          .map((car) => Carga.fromEntity(CargaEntity.fromSnapshot(car))).toList();}).listen((car) {
            cargaId = car.first.uuid;
            List<Descarga> listDescargas = List<Descarga>();
            descargasCollection.where("usuario", isEqualTo: this.usuarioId)
                .where("idCarga", isEqualTo: this.cargaId)
                .snapshots()
                .map((snapshot) {
              print(snapshot);
              return snapshot.documents
                  .map((des) => listDescargas.add(Descarga.fromEntity(DescargaEntity.fromSnapshot(des))));
            });
            streamDescargas.sink.add(listDescargas);
    });

  }

  @override
  Future<void> addNewDescarga(Descarga descarga) {
    return descargasCollection.add(descarga.copyWith(usuarioUuid: this.usuarioId).toEntity().toDocument());
  }

  @override
  Future<void> deleteDescarga(Descarga descarga) {
    return descargasCollection.document(descarga.uuid).delete();
  }

  @override
  Stream<List<Descarga>> descargas() {
    if(cargaId != '') {
      return descargasCollection.where("usuario", isEqualTo: this.usuarioId)
          .where("idCarga", isEqualTo: this.cargaId)
          .snapshots()
          .map((snapshot) {
        print(snapshot);
        return snapshot.documents
            .map((des) => Descarga.fromEntity(DescargaEntity.fromSnapshot(des)))
            .toList();
      });
    } else {
      return streamDescargas.stream;
    }
  }

  @override
  Future<void> updateDescarga(Descarga update) {
    return descargasCollection
        .document(update.uuid)
        .updateData(update.toEntity().toDocument());
  }



}