import 'dart:async';

import 'package:meta/meta.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import '../firebase_repository.dart';
import 'entities/entities.dart';

class FirebaseTropasRepository implements TropasRepository {
  CollectionReference tropasCollection;
  String empresaId;

  FirebaseTropasRepository(
      {@required String empresaId}) {
    this.tropasCollection = Firestore.instance
        .collection('empresas')
        .document(empresaId)
        .collection("rodeos");
  }

  @override
  Future<DocumentReference> addNewTropa(Tropa tropa) {
    return tropasCollection.add(tropa.toEntity().toDocument());
  }

  @override
  Future<void> deleteTropa(Tropa tropa) async {
    return tropasCollection.document(tropa.uuid).delete();
  }

  @override
  Stream<List<Tropa>> tropas() {
    return tropasCollection
        .where("activo", isEqualTo: true).where("cerrado", isEqualTo: false).snapshots().map((snapshot) {
      print(snapshot);
      return snapshot.documents
          .map((doc) => Tropa.fromEntity(TropaEntity.fromSnapshot(doc))).toList();
    });
  }

  @override
  Future<void> updateTropa(Tropa update) {
    return tropasCollection
        .document(update.uuid)
        .updateData(update.toEntity().toDocument());
  }
}
